@@ -52,7 +52,8 @@ const allCommands = {
     _configsvrRemoveShard: {skip: isPrimaryOnly},
     _configsvrRemoveShardFromZone: {skip: isPrimaryOnly},
     _configsvrRemoveTags: {skip: isPrimaryOnly},
-    _configsvrRenameCollectionMetadata: {skip: isAnInternalCommand},
+    _configsvrRepairShardedCollectionChunksHistory: {skip: isPrimaryOnly},
+    _configsvrRenameCollectionMetadata: {skip: isPrimaryOnly},
     _configsvrReshardCollection: {skip: isPrimaryOnly},
     _configsvrSetAllowMigrations: {skip: isPrimaryOnly},
     _configsvrShardCollection: {skip: isPrimaryOnly},
@@ -93,6 +94,7 @@ const allCommands = {
     _shardsvrReshardCollection: {skip: isPrimaryOnly},
     _shardsvrReshardingOperationTime: {skip: isPrimaryOnly},
     _shardsvrRefineCollectionShardKey: {skip: isPrimaryOnly},
+    _shardsvrSetAllowMigrations: {skip: isPrimaryOnly},
     _transferMods: {skip: isPrimaryOnly},
     _vectorClockPersist: {skip: isPrimaryOnly},
     abortReshardCollection: {skip: isPrimaryOnly},
@@ -278,6 +280,7 @@ const allCommands = {
     reIndex: {skip: isNotAUserDataRead},
     renameCollection: {skip: isPrimaryOnly},
     repairDatabase: {skip: isNotAUserDataRead},
+    repairShardedCollectionChunksHistory: {skip: isPrimaryOnly},
     replSetAbortPrimaryCatchUp: {skip: isNotAUserDataRead},
     replSetFreeze: {skip: isNotAUserDataRead},
     replSetGetConfig: {skip: isNotAUserDataRead},
