@@ -1,12 +1,15 @@
 /**
  * Tests that rolling back the insertion of the shardIdentity document on a shard causes the node
  * rolling it back to shut down.
- * @tags: [requires_persistence, requires_journaling]
+ * @tags: [multiversion_incompatible, requires_persistence, requires_journaling]
  */
 
 (function() {
 "use strict";
 
+// This test triggers an unclean shutdown (an fassert), which may cause inaccurate fast counts.
+TestData.skipEnforceFastCountOnValidate = true;
+
 load('jstests/libs/write_concern_util.js');
 
 var st = new ShardingTest({shards: 1});
@@ -15,14 +18,14 @@ var replTest = new ReplSetTest({nodes: 3});
 var nodes = replTest.startSet({shardsvr: ''});
 replTest.initiate();
 
+// The default WC is majority and stopServerReplication will prevent satisfying any majority writes.
+assert.commandWorked(st.s.adminCommand(
+    {setDefaultRWConcern: 1, defaultWriteConcern: {w: 1}, writeConcern: {w: "majority"}}));
+
 var priConn = replTest.getPrimary();
 var secondaries = replTest.getSecondaries();
 var configConnStr = st.configRS.getURL();
 
-// Shards start in FCV 4.0 until a config server reaches out to them. This causes storage to
-// shutdown with 4.0 compatible files, requiring rollback via refetch.
-priConn.adminCommand({setFeatureCompatibilityVersion: "4.0"});
-
 // Wait for the secondaries to have the latest oplog entries before stopping the fetcher to
 // avoid the situation where one of the secondaries will not have an overlapping oplog with
 // the other nodes once the primary is killed.
@@ -41,19 +44,20 @@ var shardIdentityDoc = {
     clusterId: ObjectId()
 };
 
-assert.writeOK(priConn.getDB('admin').system.version.update(
+assert.commandWorked(priConn.getDB('admin').system.version.update(
     {_id: 'shardIdentity'}, shardIdentityDoc, {upsert: true}));
 
 // Ensure sharding state on the primary was initialized
 var res = priConn.getDB('admin').runCommand({shardingState: 1});
 assert(res.enabled, tojson(res));
-assert.eq(shardIdentityDoc.configsvrConnectionString, res.configServer);
 assert.eq(shardIdentityDoc.shardName, res.shardName);
 assert.eq(shardIdentityDoc.clusterId, res.clusterId);
+assert.soon(() => shardIdentityDoc.configsvrConnectionString ==
+                priConn.adminCommand({shardingState: 1}).configServer);
 
 // Ensure sharding state on the secondaries was *not* initialized
 secondaries.forEach(function(secondary) {
-    secondary.setSlaveOk(true);
+    secondary.setSecondaryOk();
     res = secondary.getDB('admin').runCommand({shardingState: 1});
     assert(!res.enabled, tojson(res));
 });
@@ -75,7 +79,7 @@ restartServerReplication(secondaries);
 // Wait for a new healthy primary
 var newPriConn = replTest.getPrimary();
 assert.neq(priConn, newPriConn);
-assert.writeOK(newPriConn.getDB('test').foo.insert({a: 1}, {writeConcern: {w: 'majority'}}));
+assert.commandWorked(newPriConn.getDB('test').foo.insert({a: 1}, {writeConcern: {w: 'majority'}}));
 
 // Restart the original primary so it triggers a rollback of the shardIdentity insert. Pass
 // {waitForConnect : false} to avoid a race condition between the node crashing (which we expect)
@@ -89,8 +93,7 @@ jsTest.log("Waiting for original primary to rollback and shut down");
 // Wait until the node shuts itself down during the rollback. We will hit the first assertion if
 // we rollback using 'recoverToStableTimestamp' and the second if using 'rollbackViaRefetch'.
 assert.soon(() => {
-    return (rawMongoProgramOutput().indexOf("Fatal Assertion 50712") !== -1 ||
-            rawMongoProgramOutput().indexOf("Fatal Assertion 40498") !== -1);
+    return (rawMongoProgramOutput().search(/Fatal assertion.*(40498|50712)/) !== -1);
 });
 
 // Restart the original primary again.  This time, the shardIdentity document should already be
@@ -107,7 +110,7 @@ try {
 // specified. We do want to wait to be able to connect to the node here however, so we need to pass
 // {waitForConnect: true}.
 priConn = replTest.start(priConn.nodeId, {shardsvr: '', waitForConnect: true}, true);
-priConn.setSlaveOk();
+priConn.setSecondaryOk();
 
 // Wait for the old primary to replicate the document that was written to the new primary while
 // it was shut down.
