@@ -768,7 +768,7 @@ class NinjaState:
 
             ninja.build(**build)
 
-        template_builds = dict()
+        template_builds = {'rule': "TEMPLATE"}
         for template_builder in template_builders:
 
             # Special handling for outputs and implicit since we need to
@@ -785,9 +785,6 @@ class NinjaState:
                     new_val.append(cur_val)
                 template_builds[agg_key] = new_val
 
-            # Collect all other keys
-            template_builds.update(template_builder)
-
         if template_builds.get("outputs", []):
             ninja.build(**template_builds)
 
@@ -1087,7 +1084,7 @@ def get_command(env, node, action):  # pylint: disable=too-many-branches
     # Generate a real CommandAction
     if isinstance(action, SCons.Action.CommandGeneratorAction):
         # pylint: disable=protected-access
-        action = action._generate(tlist, slist, sub_env, 1, executor=executor)
+        action = action._generate(tlist, slist, sub_env, 0, executor=executor)
 
     variables = {}
 
