@@ -1333,7 +1333,7 @@ Status IndexCatalogImpl::_indexKeys(OperationContext* opCtx,
             }
         }
 
-        int64_t inserted;
+        int64_t inserted = 0;
         status = index->indexBuildInterceptor()->sideWrite(opCtx,
                                                            keys,
                                                            multikeyMetadataKeys,
@@ -1345,7 +1345,7 @@ Status IndexCatalogImpl::_indexKeys(OperationContext* opCtx,
             *keysInsertedOut += inserted;
         }
     } else {
-        int64_t numInserted;
+        int64_t numInserted = 0;
         status = index->accessMethod()->insertKeysAndUpdateMultikeyPaths(
             opCtx,
             coll,
@@ -1369,6 +1369,7 @@ Status IndexCatalogImpl::_indexFilteredRecords(OperationContext* opCtx,
                                                const IndexCatalogEntry* index,
                                                const std::vector<BsonRecord>& bsonRecords,
                                                int64_t* keysInsertedOut) const {
+    SharedBufferFragmentBuilder pooledBuilder(KeyString::HeapBuilder::kHeapAllocatorDefaultBytes);
     auto& executionCtx = StorageExecutionContext::get(opCtx);
 
     InsertDeleteOptions options;
@@ -1387,7 +1388,7 @@ Status IndexCatalogImpl::_indexFilteredRecords(OperationContext* opCtx,
         auto multikeyMetadataKeys = executionCtx.multikeyMetadataKeys();
         auto multikeyPaths = executionCtx.multikeyPaths();
 
-        index->accessMethod()->getKeys(executionCtx.pooledBufferBuilder(),
+        index->accessMethod()->getKeys(pooledBuilder,
                                        *bsonRecord.docPtr,
                                        options.getKeysMode,
                                        IndexAccessMethod::GetKeysContext::kAddingKeys,
@@ -1508,7 +1509,7 @@ void IndexCatalogImpl::_unindexKeys(OperationContext* opCtx,
             }
         }
 
-        int64_t removed;
+        int64_t removed = 0;
         fassert(31155,
                 index->indexBuildInterceptor()->sideWrite(
                     opCtx, keys, {}, {}, loc, IndexBuildInterceptor::Op::kDelete, &removed));
@@ -1528,7 +1529,7 @@ void IndexCatalogImpl::_unindexKeys(OperationContext* opCtx,
     // duplicates. See SERVER-17487 for more details.
     options.dupsAllowed = options.dupsAllowed || !index->isReady(opCtx, collection);
 
-    int64_t removed;
+    int64_t removed = 0;
     Status status = index->accessMethod()->removeKeys(opCtx, keys, loc, options, &removed);
 
     if (!status.isOK()) {
@@ -1552,13 +1553,14 @@ void IndexCatalogImpl::_unindexRecord(OperationContext* opCtx,
                                       const RecordId& loc,
                                       bool logIfError,
                                       int64_t* keysDeletedOut) const {
+    SharedBufferFragmentBuilder pooledBuilder(KeyString::HeapBuilder::kHeapAllocatorDefaultBytes);
     auto& executionCtx = StorageExecutionContext::get(opCtx);
 
     // There's no need to compute the prefixes of the indexed fields that cause the index to be
     // multikey when removing a document since the index metadata isn't updated when keys are
     // deleted.
     auto keys = executionCtx.keys();
-    entry->accessMethod()->getKeys(executionCtx.pooledBufferBuilder(),
+    entry->accessMethod()->getKeys(pooledBuilder,
                                    obj,
                                    IndexAccessMethod::GetKeysMode::kRelaxConstraintsUnfiltered,
                                    IndexAccessMethod::GetKeysContext::kRemovingKeys,
