@@ -458,7 +458,6 @@ boost::intrusive_ptr<ExpressionContext> makeExpressionContext(
                               uuid,
                               CurOp::get(opCtx)->dbProfileLevel() > 0);
     expCtx->tempDir = storageGlobalParams.dbpath + "/_tmp";
-    expCtx->inMultiDocumentTransaction = opCtx->inMultiDocumentTransaction();
     expCtx->collationMatchesDefault = collationMatchesDefault;
 
     return expCtx;
@@ -765,7 +764,9 @@ Status runAggregate(OperationContext* opCtx,
         expCtx = makeExpressionContext(
             opCtx, request, std::move(*collatorToUse), uuid, collatorToUseMatchesDefault);
 
+        expCtx->startExpressionCounters();
         auto pipeline = Pipeline::parse(request.getPipeline(), expCtx);
+        expCtx->stopExpressionCounters();
 
         // Check that the view's collation matches the collation of any views involved in the
         // pipeline.
@@ -871,7 +872,8 @@ Status runAggregate(OperationContext* opCtx,
             opCtx->getWriteConcern(),
             repl::ReadConcernArgs::get(opCtx),
             cmdObj,
-            privileges);
+            privileges,
+            false /*isOpQueryExhaust*/);
         if (expCtx->tailableMode == TailableModeEnum::kTailable) {
             cursorParams.setTailable(true);
         } else if (expCtx->tailableMode == TailableModeEnum::kTailableAndAwaitData) {
