@@ -36,7 +36,6 @@
 #include <vector>
 
 #include "mongo/bson/bsonobj.h"
-#include "mongo/db/catalog/index_timestamp_helper.h"
 #include "mongo/db/catalog_raii.h"
 #include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/curop.h"
@@ -54,50 +53,63 @@
 namespace mongo {
 
 MONGO_FAIL_POINT_DEFINE(hangDuringIndexBuildDrainYield);
-
-bool IndexBuildInterceptor::typeCanFastpathMultikeyUpdates(IndexType indexType) {
-    // Ensure no new indexes are added without considering whether they use the multikeyPaths
-    // vector.
-    invariant(indexType == INDEX_BTREE || indexType == INDEX_2D || indexType == INDEX_HAYSTACK ||
-              indexType == INDEX_2DSPHERE || indexType == INDEX_TEXT || indexType == INDEX_HASHED ||
-              indexType == INDEX_WILDCARD);
-    // Only BTREE indexes are guaranteed to use the multikeyPaths vector. Other index types either
-    // do not track path-level multikey information or have "special" handling of multikey
-    // information.
-    return (indexType == INDEX_BTREE);
-}
+MONGO_FAIL_POINT_DEFINE(hangDuringIndexBuildDrainYieldSecond);
+MONGO_FAIL_POINT_DEFINE(hangIndexBuildDuringDrainWritesPhase);
+MONGO_FAIL_POINT_DEFINE(hangIndexBuildDuringDrainWritesPhaseSecond);
 
 IndexBuildInterceptor::IndexBuildInterceptor(OperationContext* opCtx, IndexCatalogEntry* entry)
     : _indexCatalogEntry(entry),
       _sideWritesTable(
           opCtx->getServiceContext()->getStorageEngine()->makeTemporaryRecordStore(opCtx)),
-      _skippedRecordTracker(entry),
-      _sideWritesCounter(std::make_shared<AtomicWord<long long>>()) {
+      _skippedRecordTracker(opCtx, entry, boost::none) {
 
     if (entry->descriptor()->unique()) {
         _duplicateKeyTracker = std::make_unique<DuplicateKeyTracker>(opCtx, entry);
     }
-    // `mergeMultikeyPaths` is sensitive to the two inputs having the same multikey
-    // "shape". Initialize `_multikeyPaths` with the right shape from the IndexCatalogEntry.
-    auto indexType = entry->descriptor()->getIndexType();
-    if (typeCanFastpathMultikeyUpdates(indexType)) {
-        auto numFields = entry->descriptor()->getNumFields();
-        _multikeyPaths = MultikeyPaths{};
-        auto it = _multikeyPaths->begin();
-        _multikeyPaths->insert(it, numFields, {});
-    }
 }
 
-void IndexBuildInterceptor::deleteTemporaryTables(OperationContext* opCtx) {
-    _sideWritesTable->deleteTemporaryTable(opCtx);
-    if (_duplicateKeyTracker) {
-        _duplicateKeyTracker->deleteTemporaryTable(opCtx);
+IndexBuildInterceptor::IndexBuildInterceptor(OperationContext* opCtx,
+                                             IndexCatalogEntry* entry,
+                                             StringData sideWritesIdent,
+                                             boost::optional<StringData> duplicateKeyTrackerIdent,
+                                             boost::optional<StringData> skippedRecordTrackerIdent)
+    : _indexCatalogEntry(entry),
+      _sideWritesTable(
+          opCtx->getServiceContext()->getStorageEngine()->makeTemporaryRecordStoreFromExistingIdent(
+              opCtx, sideWritesIdent)),
+      _skippedRecordTracker(opCtx, entry, skippedRecordTrackerIdent),
+      _skipNumAppliedCheck(true) {
+
+    auto finalizeTableOnFailure = makeGuard([&] {
+        _sideWritesTable->finalizeTemporaryTable(opCtx,
+                                                 TemporaryRecordStore::FinalizationAction::kDelete);
+    });
+
+    auto dupKeyTrackerIdentExists = duplicateKeyTrackerIdent ? true : false;
+    uassert(ErrorCodes::BadValue,
+            str::stream() << "Resume info must contain the duplicate key tracker ident ["
+                          << duplicateKeyTrackerIdent
+                          << "] if and only if the index is unique: " << entry->descriptor(),
+            entry->descriptor()->unique() == dupKeyTrackerIdentExists);
+    if (duplicateKeyTrackerIdent) {
+        _duplicateKeyTracker =
+            std::make_unique<DuplicateKeyTracker>(opCtx, entry, duplicateKeyTrackerIdent.get());
     }
-    _skippedRecordTracker.deleteTemporaryTable(opCtx);
+
+    finalizeTableOnFailure.dismiss();
+}
+
+void IndexBuildInterceptor::finalizeTemporaryTables(
+    OperationContext* opCtx, TemporaryRecordStore::FinalizationAction action) {
+    _sideWritesTable->finalizeTemporaryTable(opCtx, action);
+    if (_duplicateKeyTracker) {
+        _duplicateKeyTracker->finalizeTemporaryTable(opCtx, action);
+    }
+    _skippedRecordTracker.finalizeTemporaryTable(opCtx, action);
 }
 
 Status IndexBuildInterceptor::recordDuplicateKey(OperationContext* opCtx,
-                                                 const KeyString::Value& key) {
+                                                 const KeyString::Value& key) const {
     invariant(_indexCatalogEntry->descriptor()->unique());
     return _duplicateKeyTracker->recordKey(opCtx, key);
 }
@@ -110,13 +122,11 @@ Status IndexBuildInterceptor::checkDuplicateKeyConstraints(OperationContext* opC
 }
 
 Status IndexBuildInterceptor::drainWritesIntoIndex(OperationContext* opCtx,
+                                                   const CollectionPtr& coll,
                                                    const InsertDeleteOptions& options,
                                                    TrackDuplicates trackDuplicates,
-                                                   RecoveryUnit::ReadSource readSource,
                                                    DrainYieldPolicy drainYieldPolicy) {
     invariant(!opCtx->lockState()->inAWriteUnitOfWork());
-    // Reading at a timestamp during hybrid index builds is not supported.
-    invariant(readSource == RecoveryUnit::ReadSource::kNoTimestamp);
 
     // These are used for logging only.
     int64_t totalDeleted = 0;
@@ -182,11 +192,21 @@ Status IndexBuildInterceptor::drainWritesIntoIndex(OperationContext* opCtx,
                 break;
             }
 
+            const long long iteration = _numApplied + batchSize;
+            _checkDrainPhaseFailPoint(opCtx, &hangIndexBuildDuringDrainWritesPhase, iteration);
+            _checkDrainPhaseFailPoint(
+                opCtx, &hangIndexBuildDuringDrainWritesPhaseSecond, iteration);
+
             batchSize += 1;
             batchSizeBytes += objSize;
 
-            if (auto status = _applyWrite(
-                    opCtx, unownedDoc, options, trackDuplicates, &totalInserted, &totalDeleted);
+            if (auto status = _applyWrite(opCtx,
+                                          coll,
+                                          unownedDoc,
+                                          options,
+                                          trackDuplicates,
+                                          &totalInserted,
+                                          &totalDeleted);
                 !status.isOK()) {
                 return status;
             }
@@ -222,7 +242,7 @@ Status IndexBuildInterceptor::drainWritesIntoIndex(OperationContext* opCtx,
         // Lock yielding will be directed by the yield policy provided.
         // We will typically yield locks during the draining phase if we are holding intent locks.
         if (DrainYieldPolicy::kYield == drainYieldPolicy) {
-            _yield(opCtx);
+            _yield(opCtx, &coll);
         }
 
         // Account for more writes coming in during a batch.
@@ -235,8 +255,8 @@ Status IndexBuildInterceptor::drainWritesIntoIndex(OperationContext* opCtx,
 
     // Apply batches of side writes until the last record in the table is seen.
     while (!atEof) {
-        auto swAtEof = writeConflictRetry(
-            opCtx, "index build drain", _indexCatalogEntry->ns().ns(), applySingleBatch);
+        auto swAtEof =
+            writeConflictRetry(opCtx, "index build drain", coll->ns().ns(), applySingleBatch);
         if (!swAtEof.isOK()) {
             return swAtEof.getStatus();
         }
@@ -248,17 +268,20 @@ Status IndexBuildInterceptor::drainWritesIntoIndex(OperationContext* opCtx,
     int logLevel = (_numApplied - appliedAtStart > 0) ? 0 : 1;
     LOGV2_DEBUG(20689,
                 logLevel,
-                "index build: drained side writes",
+                "Index build: drained side writes",
+                "index"_attr = _indexCatalogEntry->descriptor()->indexName(),
+                "collectionUUID"_attr = coll->uuid(),
+                logAttrs(coll->ns()),
                 "numApplied"_attr = (_numApplied - appliedAtStart),
                 "totalInserted"_attr = totalInserted,
                 "totalDeleted"_attr = totalDeleted,
-                "indexName"_attr = _indexCatalogEntry->descriptor()->indexName(),
                 "durationMillis"_attr = timer.millis());
 
     return Status::OK();
 }
 
 Status IndexBuildInterceptor::_applyWrite(OperationContext* opCtx,
+                                          const CollectionPtr& coll,
                                           const BSONObj& operation,
                                           const InsertDeleteOptions& options,
                                           TrackDuplicates trackDups,
@@ -276,15 +299,23 @@ Status IndexBuildInterceptor::_applyWrite(OperationContext* opCtx,
         (strcmp(operation.getStringField("op"), "i") == 0) ? Op::kInsert : Op::kDelete;
 
     const KeyStringSet keySet{keyString};
-    const RecordId opRecordId =
-        KeyString::decodeRecordIdAtEnd(keyString.getBuffer(), keyString.getSize());
+    const RecordId opRecordId = [&]() {
+        auto keyFormat = coll->getRecordStore()->keyFormat();
+        if (keyFormat == KeyFormat::Long) {
+            return KeyString::decodeRecordIdLongAtEnd(keyString.getBuffer(), keyString.getSize());
+        } else {
+            invariant(keyFormat == KeyFormat::String);
+            return KeyString::decodeRecordIdStrAtEnd(keyString.getBuffer(), keyString.getSize());
+        }
+    }();
 
     auto accessMethod = _indexCatalogEntry->accessMethod();
     if (opType == Op::kInsert) {
         int64_t numInserted;
         auto status = accessMethod->insertKeysAndUpdateMultikeyPaths(
             opCtx,
-            keySet,
+            coll,
+            {keySet.begin(), keySet.end()},
             {},
             MultikeyPaths{},
             opRecordId,
@@ -308,7 +339,8 @@ Status IndexBuildInterceptor::_applyWrite(OperationContext* opCtx,
             invariant(strcmp(operation.getStringField("op"), "d") == 0);
 
         int64_t numDeleted;
-        Status s = accessMethod->removeKeys(opCtx, keySet, opRecordId, options, &numDeleted);
+        Status s = accessMethod->removeKeys(
+            opCtx, {keySet.begin(), keySet.end()}, opRecordId, options, &numDeleted);
         if (!s.isOK()) {
             return s;
         }
@@ -320,9 +352,10 @@ Status IndexBuildInterceptor::_applyWrite(OperationContext* opCtx,
     return Status::OK();
 }
 
-void IndexBuildInterceptor::_yield(OperationContext* opCtx) {
+void IndexBuildInterceptor::_yield(OperationContext* opCtx, const Yieldable* yieldable) {
     // Releasing locks means a new snapshot should be acquired when restored.
     opCtx->recoveryUnit()->abandonSnapshot();
+    yieldable->yield();
 
     auto locker = opCtx->lockState();
     Locker::LockSnapshot snapshot;
@@ -332,42 +365,65 @@ void IndexBuildInterceptor::_yield(OperationContext* opCtx) {
     // Track the number of yields in CurOp.
     CurOp::get(opCtx)->yielded();
 
-    hangDuringIndexBuildDrainYield.executeIf(
-        [&](auto&&) {
-            LOGV2(20690, "Hanging index build during drain yield");
-            hangDuringIndexBuildDrainYield.pauseWhileSet();
-        },
-        [&](auto&& config) {
-            return config.getStringField("namespace") == _indexCatalogEntry->ns().ns();
-        });
+    auto failPointHang = [opCtx, indexCatalogEntry = _indexCatalogEntry](FailPoint* fp) {
+        fp->executeIf(
+            [fp](auto&&) {
+                LOGV2(20690, "Hanging index build during drain yield");
+                fp->pauseWhileSet();
+            },
+            [opCtx, indexCatalogEntry](auto&& config) {
+                return config.getStringField("namespace") ==
+                    indexCatalogEntry->getNSSFromCatalog(opCtx).ns();
+            });
+    };
+    failPointHang(&hangDuringIndexBuildDrainYield);
+    failPointHang(&hangDuringIndexBuildDrainYieldSecond);
 
     locker->restoreLockState(opCtx, snapshot);
+    yieldable->restore();
 }
 
 bool IndexBuildInterceptor::areAllWritesApplied(OperationContext* opCtx) const {
+    return _checkAllWritesApplied(opCtx, false);
+}
+
+void IndexBuildInterceptor::invariantAllWritesApplied(OperationContext* opCtx) const {
+    _checkAllWritesApplied(opCtx, true);
+}
+
+bool IndexBuildInterceptor::_checkAllWritesApplied(OperationContext* opCtx, bool fatal) const {
     invariant(_sideWritesTable);
-    auto cursor = _sideWritesTable->rs()->getCursor(opCtx);
-    auto record = cursor->next();
 
     // The table is empty only when all writes are applied.
-    if (!record) {
-        auto writesRecorded = _sideWritesCounter->load();
-        if (writesRecorded != _numApplied) {
-            dassert(writesRecorded == _numApplied,
-                    (str::stream()
-                     << "The number of side writes recorded does not match the number "
-                        "applied, despite the table appearing empty. Writes recorded: "
-                     << writesRecorded << ", applied: " << _numApplied));
-            LOGV2_WARNING(20692,
-                          "The number of side writes recorded does not match the number applied, "
-                          "despite the table appearing empty",
-                          "writesRecorded"_attr = writesRecorded,
-                          "applied"_attr = _numApplied);
-        }
+    auto cursor = _sideWritesTable->rs()->getCursor(opCtx);
+    auto record = cursor->next();
+    if (fatal) {
+        invariant(
+            !record,
+            str::stream() << "Expected all side writes to be drained but found record with id "
+                          << record->id << " and data " << record->data.toBson());
+    } else if (record) {
+        return false;
+    }
+
+    if (_skipNumAppliedCheck) {
         return true;
     }
 
-    return false;
+    auto writesRecorded = _sideWritesCounter->load();
+    if (writesRecorded != _numApplied) {
+        dassert(writesRecorded == _numApplied,
+                (str::stream() << "The number of side writes recorded does not match the number "
+                                  "applied, despite the table appearing empty. Writes recorded: "
+                               << writesRecorded << ", applied: " << _numApplied));
+        LOGV2_WARNING(20692,
+                      "The number of side writes recorded does not match the number applied, "
+                      "despite the table appearing empty",
+                      "writesRecorded"_attr = writesRecorded,
+                      "applied"_attr = _numApplied);
+    }
+
+    return true;
 }
 
 boost::optional<MultikeyPaths> IndexBuildInterceptor::getMultikeyPaths() const {
@@ -388,13 +444,13 @@ Status IndexBuildInterceptor::sideWrite(OperationContext* opCtx,
     // `multikeyMetadataKeys` when inserting.
     *numKeysOut = keys.size() + (op == Op::kInsert ? multikeyMetadataKeys.size() : 0);
 
-    auto indexType = _indexCatalogEntry->descriptor()->getIndexType();
+    // Maintain parity with IndexAccessMethod's handling of whether keys could change the multikey
+    // state on the index.
+    bool isMultikey = _indexCatalogEntry->accessMethod()->shouldMarkIndexAsMultikey(
+        keys.size(), multikeyMetadataKeys, multikeyPaths);
 
-    // No need to take the multikeyPaths mutex if this is a trivial multikey update.
-    bool canBypassMultikeyMutex = typeCanFastpathMultikeyUpdates(indexType) &&
-        MultikeyPathTracker::isMultikeyPathsTrivial(multikeyPaths);
-
-    if (op == Op::kInsert && !canBypassMultikeyMutex) {
+    // No need to take the multikeyPaths mutex if this would not change any multikey state.
+    if (op == Op::kInsert && isMultikey) {
         // SERVER-39705: It's worth noting that a document may not generate any keys, but be
         // described as being multikey. This step must be done to maintain parity with `validate`s
         // expectations.
@@ -402,10 +458,6 @@ Status IndexBuildInterceptor::sideWrite(OperationContext* opCtx,
         if (_multikeyPaths) {
             MultikeyPathTracker::mergeMultikeyPaths(&_multikeyPaths.get(), multikeyPaths);
         } else {
-            // All indexes that support pre-initialization of _multikeyPaths during
-            // IndexBuildInterceptor construction time should have been initialized already.
-            invariant(!typeCanFastpathMultikeyUpdates(indexType));
-
             // `mergeMultikeyPaths` is sensitive to the two inputs having the same multikey
             // "shape". Initialize `_multikeyPaths` with the right shape from the first result.
             _multikeyPaths = multikeyPaths;
@@ -431,7 +483,7 @@ Status IndexBuildInterceptor::sideWrite(OperationContext* opCtx,
         // from getBuffer().
         builder.reset();
         keyString.serialize(builder);
-        BSONBinData binData(builder.buf(), builder.getSize(), BinDataGeneral);
+        BSONBinData binData(builder.buf(), builder.len(), BinDataGeneral);
         toInsert.emplace_back(BSON("op" << (op == Op::kInsert ? "i" : "d") << "key" << binData));
     }
 
@@ -442,7 +494,7 @@ Status IndexBuildInterceptor::sideWrite(OperationContext* opCtx,
         for (const auto& keyString : multikeyMetadataKeys) {
             builder.reset();
             keyString.serialize(builder);
-            BSONBinData binData(builder.buf(), builder.getSize(), BinDataGeneral);
+            BSONBinData binData(builder.buf(), builder.len(), BinDataGeneral);
             toInsert.emplace_back(BSON("op"
                                        << "i"
                                        << "key" << binData));
@@ -479,9 +531,30 @@ Status IndexBuildInterceptor::sideWrite(OperationContext* opCtx,
 }
 
 Status IndexBuildInterceptor::retrySkippedRecords(OperationContext* opCtx,
-                                                  const Collection* collection) {
+                                                  const CollectionPtr& collection) {
     return _skippedRecordTracker.retrySkippedRecords(opCtx, collection);
 }
 
+void IndexBuildInterceptor::_checkDrainPhaseFailPoint(OperationContext* opCtx,
+                                                      FailPoint* fp,
+                                                      long long iteration) const {
+    fp->executeIf(
+        [=](const BSONObj& data) {
+            LOGV2(4841800,
+                  "Hanging index build during drain writes phase",
+                  "iteration"_attr = iteration,
+                  "index"_attr = _indexCatalogEntry->descriptor()->indexName());
+
+            fp->pauseWhileSet(opCtx);
+        },
+        [iteration,
+         &indexName = _indexCatalogEntry->descriptor()->indexName()](const BSONObj& data) {
+            auto indexNames = data.getObjectField("indexNames");
+            return iteration == data["iteration"].numberLong() &&
+                std::any_of(indexNames.begin(), indexNames.end(), [&indexName](const auto& elem) {
+                       return indexName == elem.String();
+                   });
+        });
+}
 
 }  // namespace mongo
