@@ -57,12 +57,6 @@ namespace mongo {
 
 using namespace fmt::literals;
 
-ShardServerProcessInterface::ShardServerProcessInterface(
-    OperationContext* opCtx, std::shared_ptr<executor::TaskExecutor> executor)
-    : CommonMongodProcessInterface(executor) {
-    _opIsVersioned = OperationShardingState::isOperationVersioned(opCtx);
-}
-
 bool ShardServerProcessInterface::isSharded(OperationContext* opCtx, const NamespaceString& nss) {
     const auto cm =
         uassertStatusOK(Grid::get(opCtx)->catalogCache()->getCollectionRoutingInfo(opCtx, nss));
@@ -200,15 +194,14 @@ void ShardServerProcessInterface::renameIfOptionsAndIndexesHaveNotChanged(
     newCmdWithWriteConcernBuilder.append(WriteConcernOptions::kWriteConcernField,
                                          opCtx->getWriteConcern().toBSON());
     newCmdObj = newCmdWithWriteConcernBuilder.done();
-    auto response = executeRawCommandAgainstDatabasePrimary(
-        opCtx,
-        // internalRenameIfOptionsAndIndexesMatch is adminOnly.
-        NamespaceString::kAdminDb,
-        cachedDbInfo,
-        // Only unsharded collections can be renamed.
-        _versionCommandIfAppropriate(newCmdObj, cachedDbInfo, ChunkVersion::UNSHARDED()),
-        ReadPreferenceSetting(ReadPreference::PrimaryOnly),
-        Shard::RetryPolicy::kNoRetry);
+    auto response =
+        executeCommandAgainstDatabasePrimary(opCtx,
+                                             // internalRenameIfOptionsAndIndexesMatch is adminOnly.
+                                             NamespaceString::kAdminDb,
+                                             std::move(cachedDbInfo),
+                                             newCmdObj,
+                                             ReadPreferenceSetting(ReadPreference::PrimaryOnly),
+                                             Shard::RetryPolicy::kNoRetry);
     uassertStatusOKWithContext(response.swResponse,
                                str::stream() << "failed while running command " << newCmdObj);
     auto result = response.swResponse.getValue().data;
@@ -220,6 +213,10 @@ void ShardServerProcessInterface::renameIfOptionsAndIndexesHaveNotChanged(
 
 BSONObj ShardServerProcessInterface::getCollectionOptions(OperationContext* opCtx,
                                                           const NamespaceString& nss) {
+    if (nss.isNamespaceAlwaysUnsharded()) {
+        return getCollectionOptionsLocally(opCtx, nss);
+    }
+
     auto cachedDbInfo =
         uassertStatusOK(Grid::get(opCtx)->catalogCache()->getDatabase(opCtx, nss.db()));
     auto shard = uassertStatusOK(
@@ -234,7 +231,7 @@ BSONObj ShardServerProcessInterface::getCollectionOptions(OperationContext* opCt
             shard->runExhaustiveCursorCommand(opCtx,
                                               ReadPreferenceSetting(ReadPreference::PrimaryOnly),
                                               nss.db().toString(),
-                                              _versionCommandIfAppropriate(cmdObj, cachedDbInfo),
+                                              appendDbVersionIfPresent(cmdObj, cachedDbInfo),
                                               Milliseconds(-1)));
     } catch (const ExceptionFor<ErrorCodes::NamespaceNotFound>&) {
         return BSONObj{};
@@ -280,7 +277,7 @@ std::list<BSONObj> ShardServerProcessInterface::getIndexSpecs(OperationContext*
             shard->runExhaustiveCursorCommand(opCtx,
                                               ReadPreferenceSetting(ReadPreference::PrimaryOnly),
                                               ns.db().toString(),
-                                              _versionCommandIfAppropriate(cmdObj, cachedDbInfo),
+                                              appendDbVersionIfPresent(cmdObj, cachedDbInfo),
                                               Milliseconds(-1)));
     } catch (ExceptionFor<ErrorCodes::NamespaceNotFound>&) {
         return std::list<BSONObj>();
@@ -297,13 +294,13 @@ void ShardServerProcessInterface::createCollection(OperationContext* opCtx,
     finalCmdBuilder.append(WriteConcernOptions::kWriteConcernField,
                            opCtx->getWriteConcern().toBSON());
     BSONObj finalCmdObj = finalCmdBuilder.obj();
-    auto response = executeRawCommandAgainstDatabasePrimary(
-        opCtx,
-        dbName,
-        cachedDbInfo,
-        _versionCommandIfAppropriate(finalCmdObj, cachedDbInfo),
-        ReadPreferenceSetting(ReadPreference::PrimaryOnly),
-        Shard::RetryPolicy::kIdempotent);
+    auto response =
+        executeCommandAgainstDatabasePrimary(opCtx,
+                                             dbName,
+                                             std::move(cachedDbInfo),
+                                             finalCmdObj,
+                                             ReadPreferenceSetting(ReadPreference::PrimaryOnly),
+                                             Shard::RetryPolicy::kIdempotent);
     uassertStatusOKWithContext(response.swResponse,
                                str::stream() << "failed while running command " << finalCmdObj);
     auto result = response.swResponse.getValue().data;
@@ -331,11 +328,11 @@ void ShardServerProcessInterface::createIndexesOnEmptyCollection(
         ns,
         "copying index for empty collection {}"_format(ns.ns()),
         [&] {
-            auto response = executeRawCommandAgainstDatabasePrimary(
+            auto response = executeCommandAgainstDatabasePrimary(
                 opCtx,
                 ns.db(),
-                cachedDbInfo,
-                _versionCommandIfAppropriate(cmdObj, cachedDbInfo),
+                std::move(cachedDbInfo),
+                cmdObj,
                 ReadPreferenceSetting(ReadPreference::PrimaryOnly),
                 Shard::RetryPolicy::kIdempotent);
 
@@ -361,14 +358,13 @@ void ShardServerProcessInterface::dropCollection(OperationContext* opCtx,
     newCmdBuilder.append(WriteConcernOptions::kWriteConcernField,
                          opCtx->getWriteConcern().toBSON());
     auto cmdObj = newCmdBuilder.done();
-    auto response = executeRawCommandAgainstDatabasePrimary(
-        opCtx,
-        ns.db(),
-        cachedDbInfo,
-        // Only unsharded collections can be dropped.
-        _versionCommandIfAppropriate(cmdObj, cachedDbInfo, ChunkVersion::UNSHARDED()),
-        ReadPreferenceSetting(ReadPreference::PrimaryOnly),
-        Shard::RetryPolicy::kIdempotent);
+    auto response =
+        executeCommandAgainstDatabasePrimary(opCtx,
+                                             ns.db(),
+                                             std::move(cachedDbInfo),
+                                             cmdObj,
+                                             ReadPreferenceSetting(ReadPreference::PrimaryOnly),
+                                             Shard::RetryPolicy::kIdempotent);
     uassertStatusOKWithContext(response.swResponse,
                                str::stream() << "failed while running command " << cmdObj);
     auto result = response.swResponse.getValue().data;
@@ -392,20 +388,10 @@ void ShardServerProcessInterface::setExpectedShardVersion(
     auto& oss = OperationShardingState::get(opCtx);
     if (oss.hasShardVersion(nss)) {
         invariant(oss.getShardVersion(nss) == chunkVersion);
-    } else if (_opIsVersioned) {
-        oss.initializeClientRoutingVersions(nss, chunkVersion, boost::none);
+    } else {
+        OperationShardingState::get(opCtx).initializeClientRoutingVersions(
+            nss, chunkVersion, boost::none);
     }
 }
 
-BSONObj ShardServerProcessInterface::_versionCommandIfAppropriate(
-    BSONObj cmdObj,
-    const CachedDatabaseInfo& cachedDbInfo,
-    boost::optional<ChunkVersion> shardVersion) {
-    if (!_opIsVersioned) {
-        return cmdObj;
-    }
-    return appendDbVersionIfPresent(
-        shardVersion ? appendShardVersion(cmdObj, *shardVersion) : cmdObj, cachedDbInfo);
-}
-
 }  // namespace mongo
