@@ -38,6 +38,7 @@
 #include "mongo/db/catalog/document_validation.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/dbdirectclient.h"
+#include "mongo/db/index_builds_coordinator.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/repl/apply_ops.h"
@@ -51,6 +52,7 @@
 #include "mongo/db/server_recovery.h"
 #include "mongo/db/session.h"
 #include "mongo/db/storage/control/journal_flusher.h"
+#include "mongo/db/storage/durable_history_pin.h"
 #include "mongo/db/storage/storage_parameters_gen.h"
 #include "mongo/db/transaction_history_iterator.h"
 #include "mongo/db/transaction_participant.h"
@@ -355,10 +357,14 @@ void ReplicationRecoveryImpl::recoverFromOplogAsStandalone(OperationContext* opC
 
     reconstructPreparedTransactions(opCtx, OplogApplication::Mode::kRecovering);
 
-    LOGV2_WARNING(21558,
-                  "Setting mongod to readOnly mode as a result of specifying "
-                  "'recoverFromOplogAsStandalone'");
-    storageGlobalParams.readOnly = true;
+    // Two-phase index builds are built in the background, which may still be in-progress after
+    // recovering from the oplog. To prevent crashing the server, skip enabling read-only mode.
+    if (IndexBuildsCoordinator::get(opCtx)->noIndexBuildInProgress()) {
+        LOGV2_WARNING(21558,
+                      "Setting mongod to readOnly mode as a result of specifying "
+                      "'recoverFromOplogAsStandalone'");
+        storageGlobalParams.readOnly = true;
+    }
 }
 
 void ReplicationRecoveryImpl::recoverFromOplogUpTo(OperationContext* opCtx, Timestamp endPoint) {
@@ -551,13 +557,19 @@ void ReplicationRecoveryImpl::_recoverFromUnstableCheckpoint(OperationContext* o
         // When `recoverFromOplog` truncates the oplog, that also happens to set the "oldest
         // timestamp" to the truncation point[1]. `_applyToEndOfOplog` will then perform writes
         // before the truncation point. Doing so violates the constraint that all updates must be
-        // timestamped newer than the "oldest timestamp". This call will move the "oldest
+        // timestamped newer than the "oldest timestamp". So we will need to move the "oldest
         // timestamp" back to the `startPoint`.
         //
+        // Before doing so, we will remove any pins. Forcing the oldest timestamp backwards will
+        // error if there are pins in place, as those pin requests will no longer be satisfied.
+        // Recovering from an unstable checkpoint has no history in the first place. Thus, clearing
+        // pins has no real effect on history being held.
+        //
         // [1] This is arguably incorrect. On rollback for nodes that are not keeping history to
         // the "majority point", the "oldest timestamp" likely needs to go back in time. The
         // oplog's `cappedTruncateAfter` method was a convenient location for this logic, which,
         // unfortunately, conflicts with the usage above.
+        DurableHistoryRegistry::get(opCtx->getServiceContext())->clearPins(opCtx);
         opCtx->getServiceContext()->getStorageEngine()->setOldestTimestamp(
             appliedThrough.getTimestamp());
 
