@@ -426,7 +426,8 @@ TEST_F(CollectionShardingRuntimeWithRangeDeleterTest,
         operationContext(),
         kTestNss,
         uuid(),
-        ChunkRange(BSON(kShardKey << MINKEY), BSON(kShardKey << MAXKEY)));
+        ChunkRange(BSON(kShardKey << MINKEY), BSON(kShardKey << MAXKEY)),
+        Milliseconds::max());
     ASSERT_EQ(status.code(), ErrorCodes::ConflictingOperationInProgress);
 }
 
@@ -441,7 +442,8 @@ TEST_F(CollectionShardingRuntimeWithRangeDeleterTest,
         opCtx,
         kTestNss,
         randomUuid,
-        ChunkRange(BSON(kShardKey << MINKEY), BSON(kShardKey << MAXKEY)));
+        ChunkRange(BSON(kShardKey << MINKEY), BSON(kShardKey << MAXKEY)),
+        Milliseconds::max());
     ASSERT_EQ(status.code(), ErrorCodes::ConflictingOperationInProgress);
 }
 
@@ -452,7 +454,11 @@ TEST_F(CollectionShardingRuntimeWithRangeDeleterTest,
     csr().setFilteringMetadata(opCtx, metadata);
 
     auto status = CollectionShardingRuntime::waitForClean(
-        opCtx, kTestNss, uuid(), ChunkRange(BSON(kShardKey << MINKEY), BSON(kShardKey << MAXKEY)));
+        opCtx,
+        kTestNss,
+        uuid(),
+        ChunkRange(BSON(kShardKey << MINKEY), BSON(kShardKey << MAXKEY)),
+        Milliseconds::max());
 
     ASSERT_OK(status);
 }
@@ -473,7 +479,11 @@ TEST_F(CollectionShardingRuntimeWithRangeDeleterTest,
 
     opCtx->setDeadlineAfterNowBy(Milliseconds(100), ErrorCodes::MaxTimeMSExpired);
     auto status = CollectionShardingRuntime::waitForClean(
-        opCtx, kTestNss, uuid(), ChunkRange(BSON(kShardKey << MINKEY), BSON(kShardKey << MAXKEY)));
+        opCtx,
+        kTestNss,
+        uuid(),
+        ChunkRange(BSON(kShardKey << MINKEY), BSON(kShardKey << MAXKEY)),
+        Milliseconds::max());
 
     ASSERT_EQ(status.code(), ErrorCodes::MaxTimeMSExpired);
 
@@ -500,7 +510,11 @@ TEST_F(CollectionShardingRuntimeWithRangeDeleterTest,
                            CollectionShardingRuntime::CleanWhen::kNow);
 
     auto status = CollectionShardingRuntime::waitForClean(
-        opCtx, kTestNss, uuid(), ChunkRange(BSON(kShardKey << MINKEY), BSON(kShardKey << MAXKEY)));
+        opCtx,
+        kTestNss,
+        uuid(),
+        ChunkRange(BSON(kShardKey << MINKEY), BSON(kShardKey << MAXKEY)),
+        Milliseconds::max());
 
     // waitForClean should block until both cleanup tasks have run. This is a best-effort check,
     // since even if it did not block, it is possible that the cleanup tasks could complete before
@@ -523,7 +537,11 @@ TEST_F(CollectionShardingRuntimeWithRangeDeleterTest,
                            CollectionShardingRuntime::CleanWhen::kNow);
 
     auto status = CollectionShardingRuntime::waitForClean(
-        opCtx, kTestNss, uuid(), ChunkRange(BSON(kShardKey << MINKEY), BSON(kShardKey << MAXKEY)));
+        opCtx,
+        kTestNss,
+        uuid(),
+        ChunkRange(BSON(kShardKey << MINKEY), BSON(kShardKey << MAXKEY)),
+        Milliseconds::max());
 
     ASSERT_OK(status);
     ASSERT(cleanupComplete.isReady());
