@@ -30,9 +30,11 @@
 #pragma once
 
 #include "mongo/base/status_with.h"
+#include "mongo/bson/bsonobj.h"
 #include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/concurrency/d_concurrency.h"
 #include "mongo/db/logical_session_cache.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/repl/optime_with.h"
 #include "mongo/executor/task_executor.h"
 #include "mongo/platform/mutex.h"
@@ -43,6 +45,7 @@
 #include "mongo/s/client/shard.h"
 #include "mongo/s/client/shard_registry.h"
 #include "mongo/s/shard_key_pattern.h"
+#include "mongo/s/write_ops/batched_command_request.h"
 
 namespace mongo {
 
@@ -184,16 +187,22 @@ public:
      * Starts and commits a transaction on the config server, with a no-op find on the specified
      * namespace in order to internally start the transaction. All writes done inside the
      * passed-in function must assume that they are run inside a transaction that will be commited
-     * after the function itself has completely finished.
+     * after the function itself has completely finished. Does not support running transaction
+     * operations remotely.
      */
     static void withTransaction(OperationContext* opCtx,
                                 const NamespaceString& namespaceForInitialFind,
                                 unique_function<void(OperationContext*, TxnNumber)> func);
+    static void withTransaction(OperationContext* opCtx,
+                                const NamespaceString& namespaceForInitialFind,
+                                unique_function<void(OperationContext*, TxnNumber)> func,
+                                const WriteConcernOptions& writeConcern);
 
     /**
      * Runs the write 'request' on namespace 'nss' in a transaction with 'txnNumber'. Write must be
      * on a collection in the config database. If expectedNumModified is specified, the number of
-     * documents modified must match expectedNumModified - throws otherwise.
+     * documents modified must match expectedNumModified - throws otherwise. Does not support
+     * running transaction operations remotely.
      */
     BSONObj writeToConfigDocumentInTxn(OperationContext* opCtx,
                                        const NamespaceString& nss,
@@ -201,14 +210,22 @@ public:
                                        TxnNumber txnNumber);
 
     /**
-     * Inserts 'docs' to namespace 'nss' in a transaction with 'txnNumber'. Breaks into multiple
-     * batches if 'docs' is larger than the max batch size. Write must be on a collection in the
-     * config database.
+     * Inserts 'docs' to namespace 'nss'. If a txnNumber is passed in, the write will be done in a
+     * transaction with 'txnNumber'. Breaks into multiple batches if 'docs' is larger than the max
+     * batch size. Write must be on a collection in the config database.
      */
-    void insertConfigDocumentsInTxn(OperationContext* opCtx,
-                                    const NamespaceString& nss,
-                                    std::vector<BSONObj> docs,
-                                    TxnNumber txnNumber);
+    void insertConfigDocuments(OperationContext* opCtx,
+                               const NamespaceString& nss,
+                               std::vector<BSONObj> docs,
+                               boost::optional<TxnNumber> txnNumber = boost::none);
+
+    /**
+     * Find a single document while under a local transaction.
+     */
+    boost::optional<BSONObj> findOneConfigDocumentInTxn(OperationContext* opCtx,
+                                                        const NamespaceString& nss,
+                                                        TxnNumber txnNumber,
+                                                        const BSONObj& query);
 
     //
     // Chunk Operations
@@ -311,6 +328,11 @@ public:
         OperationContext* opCtx,
         const NamespaceString& nss,
         unique_function<void(OperationContext*, TxnNumber)> changeMetadataFunc);
+    void bumpCollectionVersionAndChangeMetadataInTxn(
+        OperationContext* opCtx,
+        const NamespaceString& nss,
+        unique_function<void(OperationContext*, TxnNumber)> changeMetadataFunc,
+        const WriteConcernOptions& writeConcern);
 
     /**
      * Same as bumpCollectionVersionAndChangeMetadataInTxn, but bumps the version for several
@@ -320,6 +342,11 @@ public:
         OperationContext* opCtx,
         const std::vector<NamespaceString>& collNames,
         unique_function<void(OperationContext*, TxnNumber)> changeMetadataFunc);
+    void bumpMultipleCollectionVersionsAndChangeMetadataInTxn(
+        OperationContext* opCtx,
+        const std::vector<NamespaceString>& collNames,
+        unique_function<void(OperationContext*, TxnNumber)> changeMetadataFunc,
+        const WriteConcernOptions& writeConcern);
 
     /**
      * Performs a split on the chunk with min value "minKey". If the split fails, it is marked as
@@ -490,6 +517,18 @@ public:
      */
     static void clearForTests(ServiceContext* serviceContext);
 
+    //
+    // Upgrade/downgrade
+    //
+
+    /**
+     * Upgrade the chunk metadata to include the history field.
+     */
+    void upgradeChunksHistory(OperationContext* opCtx,
+                              const NamespaceString& nss,
+                              bool force,
+                              const Timestamp validAfter);
+
 private:
     /**
      * Performs the necessary checks for version compatibility and creates a new config.version
