@@ -46,6 +46,7 @@
 #include "mongo/db/session_catalog_mongod.h"
 #include "mongo/db/transaction_participant.h"
 #include "mongo/db/write_concern.h"
+#include "mongo/logv2/log.h"
 #include "mongo/logv2/redaction.h"
 #include "mongo/s/client/shard_registry.h"
 #include "mongo/s/grid.h"
@@ -55,14 +56,13 @@ namespace mongo {
 namespace {
 
 const auto kOplogField = "oplog";
-const auto kWaitsForNewOplogField = "waitsForNewOplog";
 const WriteConcernOptions kMajorityWC(WriteConcernOptions::kMajority,
                                       WriteConcernOptions::SyncMode::UNSET,
                                       Milliseconds(0));
 
 struct ProcessOplogResult {
     LogicalSessionId sessionId;
-    TxnNumber txnNum;
+    TxnNumber txnNum{kUninitializedTxnNumber};
 
     repl::OpTime oplogTime;
     bool isPrePostImage = false;
@@ -107,6 +107,23 @@ void setPrePostImageTs(const ProcessOplogResult& lastResult, repl::MutableOplogE
                           << redact(entry->toBSON()) << " to have txnNumber: " << lastResult.txnNum,
             lastResult.txnNum == entry->getTxnNumber());
 
+    // PM-2213 introduces oplog entries that link to pre/post images in the
+    // `config.image_collection` table. For chunk migration, we downconvert to the classic format
+    // where the image is stored as a no-op in the oplog. A chunk migration source will always send
+    // the appropriate no-op. This code on the destination patches up the CRUD operation oplog entry
+    // to look like the classic format.
+    if (entry->getNeedsRetryImage()) {
+        switch (entry->getNeedsRetryImage().get()) {
+            case repl::RetryImageEnum::kPreImage:
+                entry->setPreImageOpTime({repl::OpTime()});
+                break;
+            case repl::RetryImageEnum::kPostImage:
+                entry->setPostImageOpTime({repl::OpTime()});
+                break;
+        }
+        entry->setNeedsRetryImage(boost::none);
+    }
+
     if (entry->getPreImageOpTime()) {
         entry->setPreImageOpTime(lastResult.oplogTime);
     } else if (entry->getPostImageOpTime()) {
@@ -141,7 +158,7 @@ repl::MutableOplogEntry parseOplog(const BSONObj& oplogBSON) {
     uassert(ErrorCodes::UnsupportedFormat,
             str::stream() << "oplog with opTime " << oplogEntry.getTimestamp().toString()
                           << " does not have stmtId: " << redact(oplogBSON),
-            oplogEntry.getStatementId());
+            !oplogEntry.getStatementIds().empty());
 
     return oplogEntry;
 }
@@ -189,13 +206,15 @@ ProcessOplogResult processSessionOplog(const BSONObj& oplogBSON,
 
     if (oplogEntry.getOpType() == repl::OpTypeEnum::kNoop) {
         // Note: Oplog is already no-op type, no need to nest.
-        // There are two types of type 'n' oplog format expected here:
+        // There are three types of type 'n' oplog format expected here:
         // (1) Oplog entries that has been transformed by a previous migration into a
         //     nested oplog. In this case, o field contains {$sessionMigrateInfo: 1}
         //     and o2 field contains the details of the original oplog.
         // (2) Oplog entries that contains the pre/post-image information of a
         //     findAndModify operation. In this case, o field contains the relevant info
         //     and o2 will be empty.
+        // (3) Oplog entries that are a dead sentinel, which the donor sent over as the replacement
+        //     for a prepare oplog entry or unprepared transaction commit oplog entry.
 
         BSONObj object2;
         if (oplogEntry.getObject2()) {
@@ -218,7 +237,7 @@ ProcessOplogResult processSessionOplog(const BSONObj& oplogBSON,
         oplogEntry.setObject2(oplogBSON);  // TODO: strip redundant info?
     }
 
-    const auto stmtId = *oplogEntry.getStatementId();
+    const auto stmtIds = oplogEntry.getStatementIds();
 
     auto uniqueOpCtx = cc().makeOperationContext();
     auto opCtx = uniqueOpCtx.get();
@@ -229,7 +248,7 @@ ProcessOplogResult processSessionOplog(const BSONObj& oplogBSON,
 
     try {
         txnParticipant.beginOrContinue(opCtx, result.txnNum, boost::none, boost::none);
-        if (txnParticipant.checkStatementExecuted(opCtx, stmtId)) {
+        if (txnParticipant.checkStatementExecuted(opCtx, stmtIds.front())) {
             // Skip the incoming statement because it has already been logged locally
             return lastResult;
         }
@@ -240,8 +259,9 @@ ProcessOplogResult processSessionOplog(const BSONObj& oplogBSON,
             return lastResult;
         }
 
-        if (stmtId == kIncompleteHistoryStmtId) {
+        if (stmtIds.front() == kIncompleteHistoryStmtId) {
             // No need to log entries for transactions whose history has been truncated
+            invariant(stmtIds.size() == 1);
             return lastResult;
         }
 
@@ -265,10 +285,9 @@ ProcessOplogResult processSessionOplog(const BSONObj& oplogBSON,
         NamespaceString::kSessionTransactionsTableNamespace.ns(),
         [&] {
             // Need to take global lock here so repl::logOp will not unlock it and trigger the
-            // invariant that disallows unlocking global lock while inside a WUOW. Grab a DBLock
-            // here instead of plain GlobalLock to make sure the MMAPV1 flush lock will be
-            // lock/unlocked correctly. Take the transaction table db lock to ensure the same lock
-            // ordering with normal replicated updates to the table.
+            // invariant that disallows unlocking global lock while inside a WUOW. Take the
+            // transaction table db lock to ensure the same lock ordering with normal replicated
+            // updates to the table.
             Lock::DBLock lk(
                 opCtx, NamespaceString::kSessionTransactionsTableNamespace.db(), MODE_IX);
             WriteUnitOfWork wunit(opCtx);
@@ -288,9 +307,13 @@ ProcessOplogResult processSessionOplog(const BSONObj& oplogBSON,
                 sessionTxnRecord.setSessionId(result.sessionId);
                 sessionTxnRecord.setTxnNum(result.txnNum);
                 sessionTxnRecord.setLastWriteOpTime(oplogOpTime);
+
+                // Use the same wallTime as oplog since SessionUpdateTracker looks at the oplog
+                // entry wallTime when replicating.
                 sessionTxnRecord.setLastWriteDate(oplogEntry.getWallClockTime());
+
                 // We do not migrate transaction oplog entries so don't set the txn state.
-                txnParticipant.onMigrateCompletedOnPrimary(opCtx, {stmtId}, sessionTxnRecord);
+                txnParticipant.onRetryableWriteCloningCompleted(opCtx, stmtIds, sessionTxnRecord);
             }
 
             wunit.commit();
@@ -304,8 +327,10 @@ ProcessOplogResult processSessionOplog(const BSONObj& oplogBSON,
 const char SessionCatalogMigrationDestination::kSessionMigrateOplogTag[] = "$sessionMigrateInfo";
 
 SessionCatalogMigrationDestination::SessionCatalogMigrationDestination(
-    ShardId fromShard, MigrationSessionId migrationSessionId)
-    : _fromShard(std::move(fromShard)), _migrationSessionId(std::move(migrationSessionId)) {}
+    NamespaceString nss, ShardId fromShard, MigrationSessionId migrationSessionId)
+    : _nss(std::move(nss)),
+      _fromShard(std::move(fromShard)),
+      _migrationSessionId(std::move(migrationSessionId)) {}
 
 SessionCatalogMigrationDestination::~SessionCatalogMigrationDestination() {
     if (_thread.joinable()) {
@@ -319,7 +344,6 @@ void SessionCatalogMigrationDestination::start(ServiceContext* service) {
         stdx::lock_guard<Latch> lk(_mutex);
         invariant(_state == State::NotStarted);
         _state = State::Migrating;
-        _isStateChanged.notify_all();
     }
 
     _thread = stdx::thread([=] {
@@ -343,7 +367,6 @@ void SessionCatalogMigrationDestination::finish() {
     stdx::lock_guard<Latch> lk(_mutex);
     if (_state != State::ErrorOccurred) {
         _state = State::Committing;
-        _isStateChanged.notify_all();
     }
 }
 
@@ -366,8 +389,13 @@ void SessionCatalogMigrationDestination::join() {
  * 6. Wait for writes to be committed to majority of the replica set.
  */
 void SessionCatalogMigrationDestination::_retrieveSessionStateFromSource(ServiceContext* service) {
-    Client::initKillableThread("sessionCatalogMigrationProducer-" + _migrationSessionId.toString(),
-                               service);
+    Client::initThread(
+        "sessionCatalogMigrationProducer-" + _migrationSessionId.toString(), service, nullptr);
+    auto client = Client::getCurrent();
+    {
+        stdx::lock_guard lk(*client);
+        client->setSystemOperationKillableByStepdown(lk);
+    }
 
     bool oplogDrainedAfterCommiting = false;
     ProcessOplogResult lastResult;
@@ -389,7 +417,6 @@ void SessionCatalogMigrationDestination::_retrieveSessionStateFromSource(Service
 
             nextBatch = getNextSessionOplogBatch(opCtx, _fromShard, _migrationSessionId);
             oplogArray = BSONArray{nextBatch[kOplogField].Obj()};
-            const auto donorWaitsForNewOplog = nextBatch[kWaitsForNewOplogField].trueValue();
 
             if (oplogArray.isEmpty()) {
                 {
@@ -401,6 +428,13 @@ void SessionCatalogMigrationDestination::_retrieveSessionStateFromSource(Service
                         // this depleted the buffer from the source shard and receiving the commit
                         // command.
                         if (oplogDrainedAfterCommiting) {
+                            LOGV2(5087100,
+                                  "Recipient finished draining oplog entries for retryable writes "
+                                  "and transactions from donor again after receiving "
+                                  "_recvChunkCommit",
+                                  "namespace"_attr = _nss,
+                                  "migrationSessionId"_attr = _migrationSessionId,
+                                  "fromShard"_attr = _fromShard);
                             break;
                         }
 
@@ -412,35 +446,28 @@ void SessionCatalogMigrationDestination::_retrieveSessionStateFromSource(Service
                 uassertStatusOK(
                     waitForWriteConcern(opCtx, lastResult.oplogTime, kMajorityWC, &unusedWCResult));
 
-                // We depleted the buffer at least once, transition to ready for commit.
                 {
                     stdx::lock_guard<Latch> lk(_mutex);
                     // Note: only transition to "ready to commit" if state is not error/force stop.
                     if (_state == State::Migrating) {
+                        // We depleted the buffer at least once, transition to ready for commit.
+                        LOGV2(5087101,
+                              "Recipient finished draining oplog entries for retryable writes and "
+                              "transactions from donor for the first time, before receiving "
+                              "_recvChunkCommit",
+                              "namespace"_attr = _nss,
+                              "migrationSessionId"_attr = _migrationSessionId,
+                              "fromShard"_attr = _fromShard);
                         _state = State::ReadyToCommit;
-                        _isStateChanged.notify_all();
                     }
                 }
 
-                // TODO: SERVER-40187 Completely remove after v4.2. donorWaitsForNewOplog is a
-                // 'feature flag' indicating that the donor does block until there are new oplog
-                // to return so we don't need to sleep here.
-                if (!donorWaitsForNewOplog && lastOpTimeWaited == lastResult.oplogTime) {
-                    // We got an empty result at least twice in a row from the source shard so space
-                    // it out a little bit so we don't hammer the shard
-                    opCtx->sleepFor(Milliseconds(200));
-                }
-
                 lastOpTimeWaited = lastResult.oplogTime;
             }
         }
         for (BSONArrayIteratorSorted oplogIter(oplogArray); oplogIter.more();) {
             try {
                 lastResult = processSessionOplog(oplogIter.next().Obj(), lastResult);
-            } catch (const ExceptionFor<ErrorCodes::ConfigurationInProgress>&) {
-                // This means that the server has a newer txnNumber than the oplog being
-                // migrated, so just skip it
-                continue;
             } catch (const ExceptionFor<ErrorCodes::TransactionTooOld>&) {
                 // This means that the server has a newer txnNumber than the oplog being
                 // migrated, so just skip it
@@ -457,7 +484,6 @@ void SessionCatalogMigrationDestination::_retrieveSessionStateFromSource(Service
     {
         stdx::lock_guard<Latch> lk(_mutex);
         _state = State::Done;
-        _isStateChanged.notify_all();
     }
 }
 
@@ -467,11 +493,16 @@ std::string SessionCatalogMigrationDestination::getErrMsg() {
 }
 
 void SessionCatalogMigrationDestination::_errorOccurred(StringData errMsg) {
+    LOGV2(5087102,
+          "Recipient failed to copy oplog entries for retryable writes and transactions from donor",
+          "namespace"_attr = _nss,
+          "migrationSessionId"_attr = _migrationSessionId,
+          "fromShard"_attr = _fromShard,
+          "error"_attr = errMsg);
+
     stdx::lock_guard<Latch> lk(_mutex);
     _state = State::ErrorOccurred;
     _errMsg = errMsg.toString();
-
-    _isStateChanged.notify_all();
 }
 
 SessionCatalogMigrationDestination::State SessionCatalogMigrationDestination::getState() {
