@@ -295,9 +295,6 @@ void shardedRenameMetadata(OperationContext* opCtx,
         // The TO collection is not sharded or doesn't exist
     }
 
-    // Delete TO tags, even if the TO collection is not sharded or doesn't exist
-    removeTagsMetadataFromConfig_notIdempotent(opCtx, toNss, writeConcern);
-
     // Delete FROM collection entry
     deleteCollection(opCtx, fromNss, fromUUID, writeConcern);
 
@@ -323,16 +320,24 @@ void shardedRenameMetadata(OperationContext* opCtx,
         opCtx, CollectionType::ConfigNS, fromCollType.toBSON(), writeConcern));
 }
 
-void checkShardedRenamePreconditions(OperationContext* opCtx,
-                                     const NamespaceString& toNss,
-                                     const bool dropTarget) {
+void checkRenamePreconditions(OperationContext* opCtx,
+                              bool sourceIsSharded,
+                              const NamespaceString& toNss,
+                              bool dropTarget) {
+    if (sourceIsSharded) {
+        uassert(ErrorCodes::InvalidNamespace,
+                str::stream() << "Namespace of target collection too long. Namespace: " << toNss
+                              << " Max: " << NamespaceString::MaxNsShardedCollectionLen,
+                toNss.size() <= NamespaceString::MaxNsShardedCollectionLen);
+    }
+
     auto catalogClient = Grid::get(opCtx)->catalogClient();
     if (!dropTarget) {
         // Check that the sharded target collection doesn't exist
         try {
             catalogClient->getCollection(opCtx, toNss);
             // If no exception is thrown, the collection exists and is sharded
-            uasserted(ErrorCodes::CommandFailed,
+            uasserted(ErrorCodes::NamespaceExists,
                       str::stream() << "Sharded target collection " << toNss.ns()
                                     << " exists but dropTarget is not set");
         } catch (const DBException& ex) {
