@@ -169,12 +169,6 @@ public:
 
     void dropIdentForImport(OperationContext* opCtx, StringData ident) override;
 
-    Status okToRename(OperationContext* opCtx,
-                      StringData fromNS,
-                      StringData toNS,
-                      StringData ident,
-                      const RecordStore* originalRecordStore) const override;
-
     void flushAllFiles(OperationContext* opCtx, bool callerHoldsReadLock) override;
 
     Status beginBackup(OperationContext* opCtx) override;
