@@ -59,9 +59,11 @@ public:
 
         _ctx.db()->dropCollection(&_opCtx, nss()).transitional_ignore();
         _coll = _ctx.db()->createCollection(&_opCtx, nss());
+        _collPtr = _coll;
 
         ASSERT_OK(_coll->getIndexCatalog()->createIndexOnEmptyCollection(
             &_opCtx,
+            _coll,
             BSON("key" << BSON("x" << 1) << "name" << DBClientBase::genIndexName(BSON("x" << 1))
                        << "v" << static_cast<int>(kIndexVersion))));
 
@@ -85,10 +87,7 @@ public:
         PlanStage::StageState state = PlanStage::NEED_TIME;
         while (PlanStage::ADVANCED != state) {
             state = ixscan->work(&out);
-
-            // There are certain states we shouldn't get.
             ASSERT_NE(PlanStage::IS_EOF, state);
-            ASSERT_NE(PlanStage::FAILURE, state);
         }
 
         return _ws.get(out);
@@ -111,7 +110,7 @@ public:
 
         // This child stage gets owned and freed by the caller.
         MatchExpression* filter = nullptr;
-        return new IndexScan(_expCtx.get(), params, &_ws, filter);
+        return new IndexScan(_expCtx.get(), _collPtr, params, &_ws, filter);
     }
 
     IndexScan* createIndexScan(BSONObj startKey,
@@ -135,7 +134,7 @@ public:
         params.bounds.fields.push_back(oil);
 
         MatchExpression* filter = nullptr;
-        return new IndexScan(_expCtx.get(), params, &_ws, filter);
+        return new IndexScan(_expCtx.get(), _coll, params, &_ws, filter);
     }
 
     static const char* ns() {
@@ -152,6 +151,7 @@ protected:
     Lock::DBLock _dbLock;
     OldClientContext _ctx;
     Collection* _coll;
+    CollectionPtr _collPtr;
 
     WorkingSet _ws;
 
@@ -204,7 +204,7 @@ public:
         static_cast<PlanStage*>(ixscan.get())->saveState();
         insert(fromjson("{_id: 4, x: 10}"));
         insert(fromjson("{_id: 5, x: 11}"));
-        static_cast<PlanStage*>(ixscan.get())->restoreState();
+        static_cast<PlanStage*>(ixscan.get())->restoreState(&_collPtr);
 
         member = getNext(ixscan.get());
         ASSERT_EQ(WorkingSetMember::RID_AND_IDX, member->getState());
@@ -237,7 +237,7 @@ public:
         // Save state and insert an indexed doc.
         static_cast<PlanStage*>(ixscan.get())->saveState();
         insert(fromjson("{_id: 4, x: 7}"));
-        static_cast<PlanStage*>(ixscan.get())->restoreState();
+        static_cast<PlanStage*>(ixscan.get())->restoreState(&_collPtr);
 
         member = getNext(ixscan.get());
         ASSERT_EQ(WorkingSetMember::RID_AND_IDX, member->getState());
@@ -270,7 +270,7 @@ public:
         // Save state and insert an indexed doc.
         static_cast<PlanStage*>(ixscan.get())->saveState();
         insert(fromjson("{_id: 4, x: 10}"));
-        static_cast<PlanStage*>(ixscan.get())->restoreState();
+        static_cast<PlanStage*>(ixscan.get())->restoreState(&_collPtr);
 
         // Ensure that we're EOF and we don't erroneously return {'': 12}.
         WorkingSetID id;
@@ -304,7 +304,7 @@ public:
         static_cast<PlanStage*>(ixscan.get())->saveState();
         insert(fromjson("{_id: 4, x: 6}"));
         insert(fromjson("{_id: 5, x: 9}"));
-        static_cast<PlanStage*>(ixscan.get())->restoreState();
+        static_cast<PlanStage*>(ixscan.get())->restoreState(&_collPtr);
 
         // Ensure that we don't erroneously return {'': 9} or {'':3}.
         member = getNext(ixscan.get());
